@@ -24,6 +24,7 @@ import requests
 from ironic_python_agent import errors
 from ironic_python_agent.extensions import standby
 from ironic_python_agent import hardware
+from ironic_python_agent import partition_utils
 from ironic_python_agent.tests.unit import base
 from ironic_python_agent import utils
 
@@ -51,7 +52,6 @@ def _build_fake_partition_image_info():
         'ephemeral_mb': '10',
         'ephemeral_format': 'abc',
         'preserve_ephemeral': 'False',
-        'configdrive': 'configdrive',
         'image_type': 'partition',
         'boot_option': 'netboot',
         'disk_label': 'msdos',
@@ -167,25 +167,47 @@ class TestStandbyExtension(base.IronicAgentTest):
         self.assertEqual(expected_loc, location)
 
     @mock.patch('ironic_lib.disk_utils.fix_gpt_partition', autospec=True)
-    @mock.patch('builtins.open', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.trigger_device_rescan', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
-    def test_write_image(self, execute_mock, open_mock, fix_gpt_mock):
+    @mock.patch('ironic_lib.disk_utils.udev_settle', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.destroy_disk_metadata', autospec=True)
+    def test_write_image(self, wipe_mock, udev_mock, execute_mock,
+                         rescan_mock, fix_gpt_mock):
         image_info = _build_fake_image_info()
         device = '/dev/sda'
         location = standby._image_location(image_info)
-        script = standby._path_to_script('shell/write_image.sh')
-        command = ['/bin/bash', script, location, device]
-        execute_mock.return_value = ('', '')
+        command = ['qemu-img', 'convert', '-t', 'directsync',
+                   '-O', 'host_device', '-W', location, device]
 
         standby._write_image(image_info, device)
-        execute_mock.assert_called_once_with(*command)
+
+        execute_mock.assert_called_once_with(
+            *command, prlimit=mock.ANY,
+            env_variables={'MALLOC_ARENA_MAX': '3'})
+        wipe_mock.assert_called_once_with(device, '')
+        udev_mock.assert_called_once_with()
+        rescan_mock.assert_called_once_with(device)
         fix_gpt_mock.assert_called_once_with(device, node_uuid=None)
 
+    @mock.patch('ironic_lib.disk_utils.fix_gpt_partition', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.trigger_device_rescan', autospec=True)
+    @mock.patch('ironic_python_agent.utils.execute', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.udev_settle', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.destroy_disk_metadata', autospec=True)
+    def test_write_image_gpt_fails(self, wipe_mock, udev_mock, execute_mock,
+                                   rescan_mock, fix_gpt_mock):
+        image_info = _build_fake_image_info()
+        device = '/dev/sda'
+
         fix_gpt_mock.side_effect = exception.InstanceDeployFailure
         standby._write_image(image_info, device)
 
-        execute_mock.reset_mock()
-        execute_mock.return_value = ('', '')
+    @mock.patch('ironic_python_agent.utils.execute', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.udev_settle', autospec=True)
+    @mock.patch('ironic_lib.disk_utils.destroy_disk_metadata', autospec=True)
+    def test_write_image_fails(self, wipe_mock, udev_mock, execute_mock):
+        image_info = _build_fake_image_info()
+        device = '/dev/sda'
         execute_mock.side_effect = processutils.ProcessExecutionError
 
         self.assertRaises(errors.ImageWriteError,
@@ -193,14 +215,12 @@ class TestStandbyExtension(base.IronicAgentTest):
                           image_info,
                           device)
 
-        execute_mock.assert_called_once_with(*command)
-
     @mock.patch.object(utils, 'get_node_boot_mode', lambda self: 'bios')
     @mock.patch.object(hardware, 'dispatch_to_managers', autospec=True)
     @mock.patch('builtins.open', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     @mock.patch('ironic_lib.disk_utils.get_image_mb', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
     def test_write_partition_image_exception(self, work_on_disk_mock,
                                              image_mb_mock,
                                              execute_mock, open_mock,
@@ -213,7 +233,6 @@ class TestStandbyExtension(base.IronicAgentTest):
         ephemeral_format = image_info['ephemeral_format']
         node_uuid = image_info['node_uuid']
         pr_ep = image_info['preserve_ephemeral']
-        configdrive = image_info['configdrive']
         boot_mode = image_info['deploy_boot_mode']
         boot_option = image_info['boot_option']
         disk_label = image_info['disk_label']
@@ -228,14 +247,14 @@ class TestStandbyExtension(base.IronicAgentTest):
         work_on_disk_mock.side_effect = Exception_returned
 
         self.assertRaises(exc, standby._write_image, image_info,
-                          device)
+                          device, 'configdrive')
         image_mb_mock.assert_called_once_with(image_path)
         work_on_disk_mock.assert_called_once_with(device, root_mb, swap_mb,
                                                   ephemeral_mb,
                                                   ephemeral_format,
                                                   image_path,
                                                   node_uuid,
-                                                  configdrive=configdrive,
+                                                  configdrive='configdrive',
                                                   preserve_ephemeral=pr_ep,
                                                   boot_mode=boot_mode,
                                                   boot_option=boot_option,
@@ -247,7 +266,7 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch('builtins.open', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     @mock.patch('ironic_lib.disk_utils.get_image_mb', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
     def test_write_partition_image_no_node_uuid(self, work_on_disk_mock,
                                                 image_mb_mock,
                                                 execute_mock, open_mock,
@@ -261,7 +280,6 @@ class TestStandbyExtension(base.IronicAgentTest):
         ephemeral_format = image_info['ephemeral_format']
         node_uuid = image_info['node_uuid']
         pr_ep = image_info['preserve_ephemeral']
-        configdrive = image_info['configdrive']
         boot_mode = image_info['deploy_boot_mode']
         boot_option = image_info['boot_option']
         disk_label = image_info['disk_label']
@@ -276,14 +294,14 @@ class TestStandbyExtension(base.IronicAgentTest):
         image_mb_mock.return_value = 1
         work_on_disk_mock.return_value = uuids
 
-        standby._write_image(image_info, device)
+        standby._write_image(image_info, device, 'configdrive')
         image_mb_mock.assert_called_once_with(image_path)
         work_on_disk_mock.assert_called_once_with(device, root_mb, swap_mb,
                                                   ephemeral_mb,
                                                   ephemeral_format,
                                                   image_path,
                                                   node_uuid,
-                                                  configdrive=configdrive,
+                                                  configdrive='configdrive',
                                                   preserve_ephemeral=pr_ep,
                                                   boot_mode=boot_mode,
                                                   boot_option=boot_option,
@@ -297,7 +315,7 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch('builtins.open', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     @mock.patch('ironic_lib.disk_utils.get_image_mb', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
     def test_write_partition_image_exception_image_mb(self,
                                                       work_on_disk_mock,
                                                       image_mb_mock,
@@ -322,7 +340,7 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch.object(hardware, 'dispatch_to_managers', autospec=True)
     @mock.patch('builtins.open', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
     @mock.patch('ironic_lib.disk_utils.get_image_mb', autospec=True)
     def test_write_partition_image(self, image_mb_mock, work_on_disk_mock,
                                    execute_mock, open_mock, dispatch_mock):
@@ -334,7 +352,6 @@ class TestStandbyExtension(base.IronicAgentTest):
         ephemeral_format = image_info['ephemeral_format']
         node_uuid = image_info['node_uuid']
         pr_ep = image_info['preserve_ephemeral']
-        configdrive = image_info['configdrive']
         boot_mode = image_info['deploy_boot_mode']
         boot_option = image_info['boot_option']
         disk_label = image_info['disk_label']
@@ -347,14 +364,14 @@ class TestStandbyExtension(base.IronicAgentTest):
         dispatch_mock.return_value = self.fake_cpu
         work_on_disk_mock.return_value = uuids
 
-        standby._write_image(image_info, device)
+        standby._write_image(image_info, device, 'configdrive')
         image_mb_mock.assert_called_once_with(image_path)
         work_on_disk_mock.assert_called_once_with(device, root_mb, swap_mb,
                                                   ephemeral_mb,
                                                   ephemeral_format,
                                                   image_path,
                                                   node_uuid,
-                                                  configdrive=configdrive,
+                                                  configdrive='configdrive',
                                                   preserve_ephemeral=pr_ep,
                                                   boot_mode=boot_mode,
                                                   boot_option=boot_option,
@@ -577,7 +594,7 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result = self.agent_extension.cache_image(image_info=image_info)
         async_result.join()
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager', None)
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         self.assertEqual(image_info['id'],
@@ -601,10 +618,12 @@ class TestStandbyExtension(base.IronicAgentTest):
         download_mock.return_value = None
         write_mock.return_value = {'root uuid': 'root_uuid'}
         dispatch_mock.return_value = 'manager'
-        async_result = self.agent_extension.cache_image(image_info=image_info)
+        async_result = self.agent_extension.cache_image(
+            image_info=image_info, configdrive='configdrive_data')
         async_result.join()
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager',
+                                           'configdrive_data')
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         self.assertEqual(image_info['id'],
@@ -636,7 +655,7 @@ class TestStandbyExtension(base.IronicAgentTest):
         )
         async_result.join()
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager', None)
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         self.assertEqual(image_info['id'],
@@ -682,8 +701,8 @@ class TestStandbyExtension(base.IronicAgentTest):
                 autospec=True)
     @mock.patch('ironic_lib.disk_utils.list_partitions',
                 autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
                 autospec=True)
     @mock.patch('ironic_python_agent.extensions.standby._write_image',
@@ -711,7 +730,8 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result.join()
 
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager',
+                                           'configdrive_data')
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         configdrive_copy_mock.assert_called_once_with(image_info['node_uuid'],
@@ -733,8 +753,8 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     @mock.patch('ironic_lib.disk_utils.list_partitions',
                 autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
                 autospec=True)
     @mock.patch('ironic_python_agent.extensions.standby._write_image',
@@ -762,7 +782,8 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result.join()
 
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager',
+                                           'configdrive_data')
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         self.assertFalse(configdrive_copy_mock.called)
@@ -804,8 +825,8 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch('ironic_lib.disk_utils.get_disk_identifier',
                 lambda dev: 'ROOT')
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_lib.disk_utils.list_partitions',
                 autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
@@ -835,7 +856,7 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result.join()
 
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager', None)
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
 
@@ -848,9 +869,9 @@ class TestStandbyExtension(base.IronicAgentTest):
 
     @mock.patch('ironic_lib.disk_utils.get_disk_identifier',
                 lambda dev: 'ROOT')
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_lib.disk_utils.list_partitions',
                 autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
@@ -886,7 +907,7 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result.join()
 
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager', None)
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
 
@@ -899,8 +920,8 @@ class TestStandbyExtension(base.IronicAgentTest):
                 autospec=True)
     @mock.patch('ironic_lib.disk_utils.list_partitions',
                 autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
                 autospec=True)
     @mock.patch('ironic_python_agent.extensions.standby._write_image',
@@ -929,7 +950,8 @@ class TestStandbyExtension(base.IronicAgentTest):
         async_result.join()
 
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, 'manager')
+        write_mock.assert_called_once_with(image_info, 'manager',
+                                           'configdrive_data')
         dispatch_mock.assert_called_once_with('get_os_install_device',
                                               permit_refresh=True)
         configdrive_copy_mock.assert_called_once_with(image_info['node_uuid'],
@@ -952,9 +974,9 @@ class TestStandbyExtension(base.IronicAgentTest):
                 lambda _dev: [mock.Mock()])
     @mock.patch('ironic_lib.disk_utils.get_disk_identifier',
                 lambda dev: 'ROOT')
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.create_config_drive_partition',
-                autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'create_config_drive_partition',
+                       autospec=True)
     @mock.patch('ironic_python_agent.hardware.dispatch_to_managers',
                 autospec=True)
     @mock.patch('ironic_python_agent.extensions.standby.StandbyExtension'
@@ -997,7 +1019,7 @@ class TestStandbyExtension(base.IronicAgentTest):
             self.assertIs(partition, work_on_disk_mock.called)
         else:
             cache_write_mock.assert_called_once_with(mock.ANY, image_info,
-                                                     '/dev/foo')
+                                                     '/dev/foo', None)
             self.assertFalse(stream_mock.called)
 
     def test_prepare_image_raw_stream_true(self):
@@ -1049,9 +1071,10 @@ class TestStandbyExtension(base.IronicAgentTest):
 
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     def test_run_shutdown_command_valid_poweroff_sysrq(self, execute_mock):
-        execute_mock.side_effect = [('', ''), ('', ''), ('',
-                                    'Running in chroot, ignoring request.'),
-                                    ('', '')]
+        execute_mock.side_effect = [
+            ('', ''), ('', ''),
+            processutils.ProcessExecutionError(''),
+            ('', '')]
 
         self.agent_extension._run_shutdown_command('poweroff')
         calls = [mock.call('hwclock', '-v', '--systohc'),
@@ -1167,7 +1190,21 @@ class TestStandbyExtension(base.IronicAgentTest):
         device = '/dev/foo'
         self.agent_extension._cache_and_write_image(image_info, device)
         download_mock.assert_called_once_with(image_info)
-        write_mock.assert_called_once_with(image_info, device)
+        write_mock.assert_called_once_with(image_info, device, None)
+
+    @mock.patch('ironic_python_agent.extensions.standby._write_image',
+                autospec=True)
+    @mock.patch('ironic_python_agent.extensions.standby._download_image',
+                autospec=True)
+    def test_cache_and_write_image_configdirve(self, download_mock,
+                                               write_mock):
+        image_info = _build_fake_image_info()
+        device = '/dev/foo'
+        self.agent_extension._cache_and_write_image(image_info, device,
+                                                    'configdrive_data')
+        download_mock.assert_called_once_with(image_info)
+        write_mock.assert_called_once_with(image_info, device,
+                                           'configdrive_data')
 
     @mock.patch('ironic_lib.disk_utils.block_uuid', autospec=True)
     @mock.patch('ironic_lib.disk_utils.fix_gpt_partition', autospec=True)
@@ -1378,7 +1415,7 @@ class TestStandbyExtension(base.IronicAgentTest):
     @mock.patch('builtins.open', autospec=True)
     @mock.patch('ironic_python_agent.utils.execute', autospec=True)
     @mock.patch('ironic_lib.disk_utils.get_image_mb', autospec=True)
-    @mock.patch('ironic_lib.disk_utils.work_on_disk', autospec=True)
+    @mock.patch.object(partition_utils, 'work_on_disk', autospec=True)
     def test_write_partition_image_no_node_uuid_uefi(
             self, work_on_disk_mock,
             image_mb_mock,
@@ -1393,7 +1430,6 @@ class TestStandbyExtension(base.IronicAgentTest):
         ephemeral_format = image_info['ephemeral_format']
         node_uuid = image_info['node_uuid']
         pr_ep = image_info['preserve_ephemeral']
-        configdrive = image_info['configdrive']
         boot_option = image_info['boot_option']
         cpu_arch = self.fake_cpu.architecture
 
@@ -1406,14 +1442,14 @@ class TestStandbyExtension(base.IronicAgentTest):
         image_mb_mock.return_value = 1
         work_on_disk_mock.return_value = uuids
 
-        standby._write_image(image_info, device)
+        standby._write_image(image_info, device, 'configdrive')
         image_mb_mock.assert_called_once_with(image_path)
         work_on_disk_mock.assert_called_once_with(device, root_mb, swap_mb,
                                                   ephemeral_mb,
                                                   ephemeral_format,
                                                   image_path,
                                                   node_uuid,
-                                                  configdrive=configdrive,
+                                                  configdrive='configdrive',
                                                   preserve_ephemeral=pr_ep,
                                                   boot_mode='uefi',
                                                   boot_option=boot_option,
