@@ -83,7 +83,8 @@ class TestHeartbeater(ironic_agent_base.IronicAgentTest):
         time_responses.append(50)
 
         # SECOND RUN:
-        expected_stop_calls.append(mock.call(5))
+        # 50 * .5 = 25
+        expected_stop_calls.append(mock.call(25.0))
         wait_responses.append(False)
         # next heartbeat due at t=180
         heartbeat_responses.append(180)
@@ -91,11 +92,10 @@ class TestHeartbeater(ironic_agent_base.IronicAgentTest):
         uniform_responses.append(0.4)
         # time is now 80
         time_responses.append(80)
-        # add one response for _time in _heartbeat_expected
-        time_responses.append(80)
 
         # THIRD RUN:
-        expected_stop_calls.append(mock.call(5))
+        # 50 * .4 = 20
+        expected_stop_calls.append(mock.call(20.0))
         wait_responses.append(False)
         # this heartbeat attempt fails
         heartbeat_responses.append(Exception('uh oh!'))
@@ -107,7 +107,8 @@ class TestHeartbeater(ironic_agent_base.IronicAgentTest):
         time_responses.append(125.5)
 
         # FOURTH RUN:
-        expected_stop_calls.append(mock.call(5))
+        # 50 * .5 = 20
+        expected_stop_calls.append(mock.call(25.0))
         # Stop now
         wait_responses.append(True)
 
@@ -122,39 +123,6 @@ class TestHeartbeater(ironic_agent_base.IronicAgentTest):
         # Validate expectations
         self.assertEqual(expected_stop_calls,
                          self.heartbeater.stop_event.wait.call_args_list)
-        self.assertEqual(self.heartbeater.api.heartbeat.call_count, 2)
-        self.assertEqual(mock_time.call_count, 5)
-
-    @mock.patch('ironic_python_agent.agent._time', autospec=True)
-    def test__heartbeat_expected(self, mock_time):
-
-        # initial setting
-        self.heartbeater.previous_heartbeat = 0
-        self.heartbeater.interval = 0
-        self.heartbeater.heartbeat_forced = False
-        mock_time.return_value = 0
-        self.assertFalse(self.heartbeater._heartbeat_expected())
-
-        # 1st cadence
-        self.heartbeater.previous_heartbeat = 0
-        self.heartbeater.interval = 100
-        self.heartbeater.heartbeat_forced = False
-        mock_time.return_value = 5
-        self.assertFalse(self.heartbeater._heartbeat_expected())
-
-        # 2nd cadence with a forced heartbeat
-        self.heartbeater.previous_heartbeat = 0
-        self.heartbeater.interval = 100
-        self.heartbeater.heartbeat_forced = True
-        mock_time.return_value = 10
-        self.assertTrue(self.heartbeater._heartbeat_expected())
-
-        # 11th cadence with a scheduled heartbeat
-        self.heartbeater.previous_heartbeat = 0
-        self.heartbeater.interval = 100
-        self.heartbeater.heartbeat_forced = False
-        mock_time.return_value = 110
-        self.assertTrue(self.heartbeater._heartbeat_expected())
 
 
 @mock.patch.object(hardware, '_md_scan_and_assemble', lambda: None)
@@ -538,7 +506,6 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         wsgi_server.start.assert_called_once_with()
         self.agent.heartbeater.start.assert_called_once_with()
 
-    @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch('ironic_python_agent.hardware_managers.cna._detect_cna_card',
                 mock.Mock())
     @mock.patch.object(agent.IronicPythonAgent,
@@ -549,8 +516,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
     @mock.patch.object(hardware.HardwareManager, 'list_hardware_info',
                        autospec=True)
     def test_run_with_inspection(self, mock_list_hardware, mock_wsgi,
-                                 mock_dispatch, mock_inspector, mock_wait,
-                                 mock_mpath):
+                                 mock_dispatch, mock_inspector, mock_wait):
         CONF.set_override('inspection_callback_url', 'http://foo/bar')
 
         def set_serve_api():
@@ -591,7 +557,6 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                          mock_dispatch.call_args_list)
         self.agent.heartbeater.start.assert_called_once_with()
 
-    @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
@@ -609,8 +574,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                                                 mock_dispatch,
                                                 mock_inspector,
                                                 mock_wait,
-                                                mock_mdns,
-                                                mock_mpath):
+                                                mock_mdns):
         mock_mdns.side_effect = lib_exc.ServiceLookupFailure()
         # If inspection_callback_url is configured and api_url is not when the
         # agent starts, ensure that the inspection will be called and wsgi
@@ -650,7 +614,6 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         self.assertTrue(mock_wait.called)
         self.assertFalse(mock_dispatch.called)
 
-    @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch('ironic_lib.mdns.get_endpoint', autospec=True)
     @mock.patch(
         'ironic_python_agent.hardware_managers.cna._detect_cna_card',
@@ -668,8 +631,7 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
                                                mock_dispatch,
                                                mock_inspector,
                                                mock_wait,
-                                               mock_mdns,
-                                               mock_mpath):
+                                               mock_mdns):
         mock_mdns.side_effect = lib_exc.ServiceLookupFailure()
         # If both api_url and inspection_callback_url are not configured when
         # the agent starts, ensure that the inspection will be skipped and wsgi
@@ -780,9 +742,6 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         expected_result = {
             'id': result.id,
             'command_name': 'foo_command',
-            'command_params': {
-                'fail': False,
-            },
             'command_status': 'RUNNING',
             'command_result': None,
             'command_error': None,
@@ -804,9 +763,6 @@ class TestBaseAgent(ironic_agent_base.IronicAgentTest):
         expected_result = {
             'id': result.id,
             'command_name': 'foo_command',
-            'command_params': {
-                'fail': True,
-            },
             'command_status': 'RUNNING',
             'command_result': None,
             'command_error': None,
@@ -1000,13 +956,12 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertFalse(mock_exec.called)
         self.assertFalse(mock_gethostbyname.called)
 
-    @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch.object(netutils, 'get_ipv4_addr',
                        autospec=True)
     @mock.patch('ironic_python_agent.hardware_managers.cna._detect_cna_card',
                 autospec=True)
-    def test_with_network_interface(self, mock_cna, mock_get_ipv4, mock_mpath,
-                                    mock_exec, mock_gethostbyname):
+    def test_with_network_interface(self, mock_cna, mock_get_ipv4, mock_exec,
+                                    mock_gethostbyname):
         self.agent.network_interface = 'em1'
         mock_get_ipv4.return_value = '1.2.3.4'
         mock_cna.return_value = False
@@ -1019,14 +974,12 @@ class TestAdvertiseAddress(ironic_agent_base.IronicAgentTest):
         self.assertFalse(mock_exec.called)
         self.assertFalse(mock_gethostbyname.called)
 
-    @mock.patch.object(hardware, '_enable_multipath', autospec=True)
     @mock.patch.object(netutils, 'get_ipv4_addr',
                        autospec=True)
     @mock.patch('ironic_python_agent.hardware_managers.cna._detect_cna_card',
                 autospec=True)
     def test_with_network_interface_failed(self, mock_cna, mock_get_ipv4,
-                                           mock_mpath, mock_exec,
-                                           mock_gethostbyname):
+                                           mock_exec, mock_gethostbyname):
         self.agent.network_interface = 'em1'
         mock_get_ipv4.return_value = None
         mock_cna.return_value = False
